﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

/*
 * File Renaming Wizard copyright (c) 2014 Matthew Schlachter
 * See LICENSE.txt for distribution and modification restrictions
 * 
 * http://www.webmann.info/
 */
namespace FileRenamingWizard
{
    /// <summary>
    /// Interaction logic for ProcessingWindow.xaml
    /// </summary>
    public partial class ProcessingWindow : Window
    {
        public FileRenamingManager manager;
        /// <summary>
        /// The location of the file containing the default owner for the user
        /// </summary>
        private string ownerFile = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\FileRenamerOwner.txt";

        /// <summary>
        /// Does the initial loading of the processing window
        /// </summary>
        /// <param name="man">The file renaming manager containing the list of filenames</param>
        public ProcessingWindow(FileRenamingManager man)
        {
            InitializeComponent();
            manager = man;

            //==[ Initialize the values for the combo boxes
            cbxType.ItemsSource = new string[] { "Custom", "Accommodation", "Admin", "Agenda", "Agreement", "Assessment", "Backgrounder", "BestPractice", "Briefing", "BriefingMinESDC", "BriefingMinLabour", "BriefingDMESDC", "BriefingDMLabour", "BriefingDHSC", "BriefingADM", "BriefingSOS", "Brochure", "Budget", "BusinessCase", "BCP", "BusinessNeed", "BusinessRequirement", "Channel", "Communication", "Contract", "Correspondence", "DataAndStats", "Directive", "Enquiry", "Event", "Financial", "ForeignBenefit", "Form", "Framework", "GraphicalMaterial", "GuidelineProcedure", "HR", "Info", "InfoKit", "Inquiry", "Invoice", "Itinerary", "Legal", "List", "Log", "Manual", "MTP", "MediaAdvisory", "MediaLine", "Meeting", "Memo", "MoU", "Minutes", "Newsletter", "NewsRelease", "Outline", "Plan", "Policy", "PolicyRefMaterial", "Presentation", "Procedure", "Proposal", "Publication", "QPCard or QPNote", "QuestionandAnswers", "ROD", "Reference", "ReleaseNote", "Report", "ScenarioNote", "Schedule", "Scope", "Security", "Speech", "SpeakingNotes", "Signature", "SOMC", "Standard", "Submission", "Survey", "Systems", "Template", "TOR", "TestScript", "Training", "Transcode", "User Need", "UserRequirement", "WebUpdate", "WorkDescription", "WorkPlan", "WWW" };
            cbxLanguage.ItemsSource = new string[] { "", "-E", "-F", "-Bil" };
            cbxLanguage.SelectedIndex = 0;

            //==[ Fetch the owner from the owner file
            txtOwner.Text = getOwner();

            //==[ Do the initial load of the first file
            this.loadFile();
        }

        /// <summary>
        /// Initialize the fields for the current file
        /// </summary>
        private void loadFile()
        {
            lblOldName.Content = manager.getOldFileName();
            lblIndex.Content = (manager.curIndex + 1) + "/" + manager.numFiles;
            txtDate.Text = manager.getDate();
            txtDescription.Clear();

            //==[ Load the preview
            uctrPreview.Visibility = System.Windows.Visibility.Hidden;
            pnlOpen.Visibility = System.Windows.Visibility.Hidden;
            //==[ We can display word, text, and html files it seems; 
            //==[ it skips everything else to save processing time
            if (new Regex(".doc|.txt|.htm").IsMatch(manager.getOldFileName()))
            {
                try
                {
                    uctrPreview.LoadDocument(manager.files[manager.curIndex]);
                    uctrPreview.Visibility = System.Windows.Visibility.Visible;
                }
                catch
                {
                    //==[ If the preview can't be loaded show the Open button
                    pnlOpen.Visibility = System.Windows.Visibility.Visible;
                }
            }
            else
            {
                //==[ If it has an unwanted extension show the open button instead of a preview
                pnlOpen.Visibility = System.Windows.Visibility.Visible;
            }

            //==[ Automatically guess at the file type (an educated guess, really)
            cbxType.SelectedIndex = -1;
            txtType.Visibility = System.Windows.Visibility.Collapsed;
            try
            {
                cbxType.SelectedIndex = manager.getTypeIndex();
            }
            catch { }
        }


        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            //==[ Check presence of owner
            if (txtOwner.Text.Length == 0)
            {
                MessageBox.Show("You must enter an owner");
                return;
            }
            //==[ Check validity of owner
            if (!new Regex("^[A-Z]{1,10}$").IsMatch(txtOwner.Text))
            {
                MessageBox.Show("The owner must exclusively consist of 1-10 uppercase letters");
                return;
            }
            //==[ Check presence of type
            if (cbxType.SelectedIndex == -1)
            {
                MessageBox.Show("You must select a type from the dropdown menu");
                return;
            }
            //==[ Check validity of custom type
            else if (cbxType.SelectedValue.Equals("Custom") && !new Regex("^[a-zA-Z0-9]+$").IsMatch(txtType.Text))
            {
                MessageBox.Show("If Custom is selected as the type then a type must be entered containing only letters and numbers (no hyphens, spaces, or accented letters)");
                return;
            }
            //==[ Check presence and validity of description
            if (!new Regex("^[a-zA-Z0-9]+$").IsMatch(txtDescription.Text))
            {
                MessageBox.Show("A file description must be entered containing only letters and numbers (no hyphens, spaces, or accented letters)");
                return;
            }
            //==[ Check presence and validity of date
            if (!new Regex("^[0-9]{4,}[A-Za-z]{3,}[0-9]{2,}$").IsMatch(txtDate.Text))
            {
                MessageBox.Show("Date must contain four numeric digits for the year, an abreviated month, and two more digits for the day");
                return;
            }
            //==[ Check presence and validity of version
            if (!Regex.IsMatch(txtVersion.Text, "^([0-9])|([0-9][0-9])|(Final)$", RegexOptions.IgnoreCase))
            {
                MessageBox.Show("Version must either be a two-digit number or the word 'Final'");
                return;
            }
            //==[ If version is one digit long prepend a 0
            if (txtVersion.Text.Length == 1) txtVersion.Text = "0" + txtVersion.Text;
            string version = txtVersion.Text;
            //==[ If more than two digits then version is Final, otherwise prepend a V
            if (version.Length > 2)
                version = "Final";
            else
                version = "V" + version;
            //==[ Fetch the type either from the combo box or the custom type field
            string type = cbxType.SelectedValue.ToString();
            if (cbxType.SelectedValue.Equals("Custom"))
                type = txtType.Text;

            //==[ Attempt to rename the file and move to the next
            try
            {
                manager.rename(txtOwner.Text + "-" + type + "-" + txtDescription.Text + "-" + txtDate.Text + cbxLanguage.SelectedValue.ToString() + "-" + version);
                saveOwner(txtOwner.Text);
                moveToNext();
            }
            catch (Exception)
            {
                MessageBox.Show("An error occurred renaming the file. The file may be open or in use by another program.");
            }
        }

        /// <summary>
        /// Fetch the owner from the user's owner file, default PPSB if missing or invalid
        /// </summary>
        /// <returns>The owner</returns>
        private string getOwner()
        {
            string owner;
            try
            {
                owner = System.IO.File.ReadAllText(ownerFile).Trim();
            }
            catch
            {
                owner = null;
            }
            if (owner == null || !new Regex("^[A-Z]{1,10}$").IsMatch(owner))
                return "PPSB";
            else
                return owner;
        }

        /// <summary>
        /// Update the owner file with the current owner
        /// </summary>
        /// <param name="owner">The current owner</param>
        private void saveOwner(string owner)
        {
            try
            {
                System.IO.File.WriteAllText(ownerFile, owner);
            }
            catch { }
        }

        /// <summary>
        /// Skip the current file and go to the next
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSkip_Click(object sender, RoutedEventArgs e)
        {
            manager.skip();
            moveToNext();
        }

        /// <summary>
        /// Move to the next file
        /// </summary>
        private void moveToNext()
        {
            if (manager.hasNext)
                loadFile();
            // new ProcessingWindow(manager).Show();
            else
            {
                MessageBox.Show(manager.numRenamed + " files have been renamed\n" + manager.numSkipped + " files have been skipped");
                // Display finished window
                this.Close();
            }
        }

        /// <summary>
        /// Open the current file for viewing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Process.Start(manager.files[manager.curIndex]);
        }

        /// <summary>
        /// If the selection changes to 'Custom' then display the custom type field,
        /// otherwise hide it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (cbxType.SelectedIndex > -1)
            {
                if (cbxType.SelectedValue.Equals("Custom"))
                    txtType.Visibility = System.Windows.Visibility.Visible;
                else txtType.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
    }
}
